/*
 * PokeFrog - A 'Pokemonized' verison of Frogger, the well-known Konami game.
 *
 * copyright (c) 2012 Fabien LOISON <http://www.flogisoft.com/>
 * 
 * this program is free software. it comes without any warranty, to
 * the extent permitted by applicable law. you can redistribute it
 * and/or modify it under the terms of the do what the fuck you want
 * to public license, version 2, as published by sam hocevar. see
 * http://sam.zoy.org/wtfpl/copying for more details. 
 */


/**
 * @module PokeFrog
 */


/**
 * Manages all the game.
 *
 * @class Game
 */
function Game()
{
	// Tilesets
	this._tsMain = new wjs.Tileset(
			wjs.loader.getImage('main'),
			RES_TILESET_MAIN);
	this._tsPokemon = new wjs.Tileset(
			wjs.loader.getImage('pokemon'),
			RES_TILESET_POKEMON);
	this._tsEnemies = new wjs.Tileset(
			wjs.loader.getImage('enemies'),
			RES_TILESET_ENEMIES);

	// Background map
	this._mapBg = new wjs.Map(this._tsMain, 32, 32, RES_MAP_BG);

	// Lilies
	this._lilies = [
		new Lily(this._tsMain, 32, 0, 64, 64),
		new Lily(this._tsMain, 160, 0, 64, 64),
		new Lily(this._tsMain, 288, 0, 64, 64),
		new Lily(this._tsMain, 416, 0, 64, 64),
		new Lily(this._tsMain, 544, 0, 64, 64)];

	// Player
	this.player = null;
	this._resetPlayer();

	// Enemies
	this._enemies = [];
	for (var i=0 ; i<5 ; i++)
	{
		this._enemies.push({
			'orientation': (i%2) ? 'left' : 'right',
			'speed': Math.floor((Math.random()*3)+1),
			'next': 0,
			'y': 253 + 32 * i - i * 2,
			'enemies': []
		});
	}

	// Lives
	this._lives = 3;

	// Game finished (win or gameover)
	this._isFinished = false;
	this._finishScreenCb = null;

	// Register the refresh method on the main loop of wjs.
	wjs.mainLoop.addCallback('game', this.refresh.bind(this));
}

/**
 * End the game.
 *
 * @method end
 */
Game.prototype.end = function()
{
	wjs.mainLoop.delCallback('game');
	if (this.player)
	{
		this.player.die();
		this.player = null;
	}
	game = null;
}

/**
 * The player die (live-1,...)
 *
 * @method playerDie
 */
Game.prototype.playerDie = function()
{
	this._lives -= 1;
	if (this._lives <= 0)
	{
		this._lives = 0;
	}
	this._resetPlayer();
}

/**
 * Refresh the game state and the canvas.
 *
 * @method refresh
 */
Game.prototype.refresh = function()
{
	if (!this._isFinished)
	{
		this.update();
	}
	this.draw();
	if (this._isFinished && this._finishScreenCb)
	{
		this._finishScreenCb();
	}
}

/**
 * Update the game state (player'move, enemies, game over,...)
 *
 * @method update
 */
Game.prototype.update = function()
{
	// Update player
	// TODO

	// Update enemies
	for (var line in this._enemies)
	{
		// New enemy
		this._enemies[line].next -= 1;
		if (this._enemies[line].next <= 0 && wjs.helpers.arrayLength(this._enemies[line].enemies) < 6)
		{
			this._enemies[line].next = Math.floor((Math.random()*180)+80); // FIXME random
			this._enemies[line].enemies.push(new Enemy(
						this._tsEnemies,
						'truck-' + this._enemies[line].orientation,
						this._enemies[line].orientation,
						this._enemies[line].speed,
						this._enemies[line].y,
						[0, 10, 64, 32]
						));	
		}
		// Update enemies
		for (var i in this._enemies[line].enemies)
		{
			this._enemies[line].enemies[i].update();
			// Check for collide with player
			if (wjs.collide.checkRectRect(this.player.getHitbox(), this._enemies[line].enemies[i].getHitbox()))
			{
				this.playerDie();
			}
			// Delete enemies that are out of the screen.
			if (this._enemies[line].enemies[i].outOfScreen)
			{
				delete this._enemies[line].enemies[i];
			}
		}
	}

	// Check collides
	this._checkCollideLilies();

	// Check end of game
	this._checkGameOver();
	this._checkWin();

	// Update tilesets
	this._tsMain.update();
	this._tsPokemon.update();
	this._tsEnemies.update();
}

/**
 * Draw all the game (background, player, enemies,...) on the canvas.
 *
 * @method draw
 */
Game.prototype.draw = function()
{
	// Draw map bg
	this._mapBg.draw();

	// Draw lilies
	for (var i in this._lilies)
	{
		this._lilies[i].draw();
	}

	// Draw enemies
	for (var line in this._enemies)
	{
		for (var i in this._enemies[line].enemies)
		{
			this._enemies[line].enemies[i].draw();
		}
	}

	// Draw player
	if (this.player)
	{
		this.player.draw();
	}

	// Draw lives
	this._tsMain.blit('darkbg0', 0, 416);
	this._tsMain.blit('lives'+this._lives, 0, 416);
}

/**
 * Check if the player lost.
 *
 * @method _checkGameOver
 */
Game.prototype._checkGameOver = function()
{
	if (this._lives <= 0)
	{
		this._isFinished = true;
		this.player.die();
		
		var fs = new FinishScreen(
				wjs.loader.getImage('gameoverscreen'),
				function() {
					game.end();
					game = new Game();
				});
		this._finishScreenCb = fs.draw.bind(fs);
	}
}

/**
 * Check if the player won.
 *
 * @method _checkWin
 */
Game.prototype._checkWin = function()
{
	var count = 0;

	for (var i in this._lilies)
	{
		if (this._lilies[i].taken)
		{
			count += 1;
		}
		else
		{
			return;
		}
	}

	if (count == this._lilies.length)
	{
		this._isFinished = true;
		this.player.die();
		
		var fs = new FinishScreen(
				wjs.loader.getImage('winscreen'),
				function() {
					game.end();
					game = new Game();
				});
		this._finishScreenCb = fs.draw.bind(fs);
	}
}

/**
 * Check collides with lilies and update the games status.
 *
 * @method _checkCollideLilies
 * @private
 */
Game.prototype._checkCollideLilies = function()
{
	for (var i in this._lilies)
	{
		if (wjs.collide.checkRectRect(this.player.getHitbox(), this._lilies[i].getHitbox()))
		{
			if (this._lilies[i].taken)
			{
				this.playerDie();
			}
			else
			{
				this._lilies[i].taken = true;
				this._resetPlayer();
			}
		}
	}
}

/**
 * Kill the current player if exists, and create a new one at the start point.
 *
 * @method _resetPlayer
 * @private
 */
Game.prototype._resetPlayer = function()
{
	if (this.player)
	{
		this.player.die();
	}
	this.player = new Player(this._tsPokemon, RES_HITBOX_POKEMON);
}
